"""
Created on 5 Apr 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

import json

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.security.cognito_device import CognitoDeviceIdentity


# --------------------------------------------------------------------------------------------------------------------

class CognitoDeviceFinder(APIClient):
    """
    classdocs
    """

    __URL = 'https://6c2sfqt656.execute-api.us-west-2.amazonaws.com/default/CognitoDevices/retrieve'

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def find_all(self, token):
        url = '/'.join((self.__URL, 'all'))

        response = self._http_client.get(url, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoDeviceIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def find_by_tag(self, token, tag):
        url = '/'.join((self.__URL, 'in'))
        payload = json.dumps({"username": tag})

        response = self._http_client.get(url, data=payload, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoDeviceIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def get_by_tag(self, token, tag):
        url = '/'.join((self.__URL, 'exact'))
        payload = json.dumps({"username": tag})

        response = self._http_client.get(url, data=payload, headers=self._token_headers(token))
        self._check_response(response)

        return CognitoDeviceIdentity.construct_from_jdict(response.json())


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CognitoDeviceFinder:{}"


# --------------------------------------------------------------------------------------------------------------------

class CognitoDeviceIntrospector(APIClient):
    """
    classdocs
    """

    __URL = 'https://6c2sfqt656.execute-api.us-west-2.amazonaws.com/default/CognitoDevices/self'

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def find_self(self, token):
        response = self._http_client.get(self.__URL, headers=self._token_headers(token))
        self._check_response(response)

        return CognitoDeviceIdentity.construct_from_jdict(response.json())


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CognitoDeviceIntrospector:{}"



"""
Created on 25 Dec 2018

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

Equivalent to cURLs:
curl "https://aws.southcoastscience.com/device-topics?topic=south-coast-science-dev/alphasense/loc/303/gases"
curl "https://aws.southcoastscience.com/device-topics?device=scs-bgx-303"
"""

from urllib.parse import parse_qs, urlparse

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.data.byline import Byline, DeviceBylineGroup, TopicBylineGroup


# --------------------------------------------------------------------------------------------------------------------

class BylineManager(APIClient):
    """
    classdocs
    """

    # __URL = 'https://aws.southcoastscience.com/device-topics'
    __URL = 'https://cxzne688y0.execute-api.us-west-2.amazonaws.com/default/Bylines'

    __DEVICE =      'device'
    __TOPIC =       'topic'


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client, reporter=None):
        super().__init__(http_client)

        self.__reporter = reporter                              # BatchDownloadReporter


    # ----------------------------------------------------------------------------------------------------------------

    def find_latest_byline_for_topic(self, topic):
        params = {self.__TOPIC: topic}

        response = self._http_client.get(self.__URL, params=params)
        jdict = response.json()

        # bylines...
        if jdict is None:
            return None

        latest_byline = None

        for item in jdict:
            byline = Byline.construct_from_jdict(item)

            if latest_byline is None or latest_byline.rec < byline.rec:
                latest_byline = byline

        return latest_byline


    def find_bylines(self, excluded=None):
        params = {}
        items_jdict = []

        while True:
            response = self._http_client.get(self.__URL, params=params)
            jdict = response.json()

            block = jdict.get('Items')
            items_jdict += block

            # report...
            if self.__reporter:
                self.__reporter.print(len(block))

            # next...
            if jdict.get('next') is None:
                break

            next_url = urlparse(jdict.get('next'))
            params = parse_qs(next_url.query)

        # bylines...
        return TopicBylineGroup.construct_from_jdict(items_jdict, excluded=excluded, skeleton=True)


    def find_bylines_for_topic(self, topic, excluded=None):
        params = {self.__TOPIC: topic}

        response = self._http_client.get(self.__URL, params=params)
        self._check_response(response)

        jdict = response.json()

        # bylines...
        return TopicBylineGroup.construct_from_jdict(jdict, excluded=excluded, skeleton=True)


    def find_bylines_for_device(self, device, excluded=None):
        params = {self.__DEVICE: device}

        response = self._http_client.get(self.__URL, params=params)
        self._check_response(response)

        jdict = response.json()

        # bylines...
        return DeviceBylineGroup.construct_from_jdict(jdict, excluded=excluded, skeleton=True)


    def find_byline_for_device_topic(self, device, topic):
        params = {self.__DEVICE: device}

        response = self._http_client.get(self.__URL, params=params)
        self._check_response(response)

        jdict = response.json()

        # bylines...
        if jdict is None:
            return None

        for item in jdict:
            byline = Byline.construct_from_jdict(item)

            if byline.topic == topic:
                return byline

        return None


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BylineManager:{reporter:%s}" % self.__reporter



"""
Created on 19 Apr 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from abc import ABC
from http import HTTPStatus

from scs_core.client.http_exception import HTTPException
from scs_core.sys.logging import Logging


# --------------------------------------------------------------------------------------------------------------------

class APIClient(ABC):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        """
        Constructor
        """
        self.__http_client = http_client                        # requests package
        self.__logger = Logging.getLogger()


    # ----------------------------------------------------------------------------------------------------------------

    def _auth_headers(self, auth):
        header = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/json", "Authorization": auth}
        self.__logger.debug('headers: %s' % header)

        return header


    def _token_headers(self, token):
        header = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain", "Token": token}
        self.__logger.debug('headers: %s' % header)

        return header


    def _check_response(self, response):
        self.__logger.debug('response.json: %s' % response.json())

        status = HTTPStatus(response.status_code)

        if status != HTTPStatus.OK:
            raise HTTPException.construct(status.value, response.reason, response.json())


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def _http_client(self):
        return self.__http_client


    @property
    def _logger(self):
        return self.__logger


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return self.__class__.__name__ + ":{http_client:%s}" % self._http_client



"""
Created on 20 Sep 2022

@author: Jade Page (jade.page@southcoastscience.com)

Helper class for the bylines lambda
"""

import json

from collections import OrderedDict
from http import HTTPStatus

from scs_core.aws.data.http_response import HTTPResponse
from scs_core.client.http_exception import HTTPException


# ----------------------------------------------------------------------------------------------------------------

class BylineExclusiveStartKey(object):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_qsp(cls, qsp):
        if not qsp:
            return None

        device = qsp.get('device')
        topic = qsp.get('topic')

        return cls(device, topic)


    @classmethod
    def construct_from_jdict(cls, jdict):
        return cls.construct_from_qsp(jdict)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, device, topic):
        """
        Constructor
        """
        self.__device = device                                      # string
        self.__topic = topic                                        # string


    # ----------------------------------------------------------------------------------------------------------------

    def params(self):
        params = {
            'device': self.device,
            'topic': self.topic
        }

        return params


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def device(self):
        return self.__device


    @property
    def topic(self):
        return self.__topic


# --------------------------------------------------------------------------------------------------------------------


class BylineRequest(object):
    """
    classdocs
    """
    TAG_FILTER = 'device'
    TOPIC_FILTER = 'topic'
    EXCLUSIVE_START_KEY = 'exclusiveStartKey'

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_qsp(cls, qsp):
        if not qsp:
            return None

        tag_filter = qsp.get(cls.TAG_FILTER)
        topic_filter = qsp.get(cls.TOPIC_FILTER)

        esk_json = qsp.get(cls.EXCLUSIVE_START_KEY)
        exclusive_start_key = BylineExclusiveStartKey.construct_from_qsp(json.loads(esk_json)) if esk_json else None

        return cls(tag_filter, topic_filter, exclusive_start_key=exclusive_start_key)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, tag_filter, topic_filter, exclusive_start_key=None):
        """
        Constructor
        """
        self.__tag_filter = tag_filter  # string
        self.__topic_filter = topic_filter  # string

        self.__exclusive_start_key = exclusive_start_key  # ExclusiveStartKey


    # ----------------------------------------------------------------------------------------------------------------

    def params(self):
        params = {}

        if self.tag_filter is not None:
            params[self.TAG_FILTER] = self.tag_filter

        if self.topic_filter is not None:
            params[self.TOPIC_FILTER] = self.__topic_filter

        if self.exclusive_start_key is not None:
            params[self.EXCLUSIVE_START_KEY] = json.dumps(self.exclusive_start_key.params())

        return params

    # ----------------------------------------------------------------------------------------------------------------

    @property
    def tag_filter(self):
        return self.__tag_filter


    @property
    def topic_filter(self):
        return self.__topic_filter


    @property
    def exclusive_start_key(self):
        return self.__exclusive_start_key


    @exclusive_start_key.setter
    def exclusive_start_key(self, exclusive_start_key):
        self.__exclusive_start_key = exclusive_start_key


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "ConfigurationRequest:{tag_filter:%s, topic_filter:%s, exclusive_start_key:%s}" % \
               (self.tag_filter, self.topic_filter, self.exclusive_start_key)


class BylinesResponse(HTTPResponse):
    """
    classdocs
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        status = HTTPStatus(jdict.get('statusCode'))

        if status != HTTPStatus.OK:
            raise HTTPException.construct(status.value, status.phrase, status.description)

        items = jdict.get('Items')
        next_url = jdict.get('next')

        return cls(status, items, next_url=next_url)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, status, items, next_url=None):
        """
        Constructor
        """
        super().__init__(status)

        self.__items = items  # list of string
        self.__next_url = next_url  # URL string


    def __len__(self):
        return len(self.items)


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['statusCode'] = self.status.value

        if self.items is not None:
            jdict['Items'] = self.items
            jdict['itemCount'] = len(self.items)

        if self.next_url is not None:
            jdict['next'] = self.next_url

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def items(self):
        return self.__items


    @property
    def next_url(self):
        return self.__next_url


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "BylinesResponse:{status:%s, items:%s, next_url:%s}" % \
               (self.status, self.items, self.next_url)



"""
Created on 17 Jun 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.aws.client.api_client import APIClient

from scs_core.aws.data.device_monitor_report import DeviceMonitorReport


# --------------------------------------------------------------------------------------------------------------------

class DeviceMonitorStatusManager(APIClient):
    """
    classdocs
    """

    __URL = "https://0l7fwqkzr8.execute-api.us-west-2.amazonaws.com/default/DeviceMonitorStatus"

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def find(self, token, device_tag_filter=None, exact=False):
        payload = {
            'device_tag': device_tag_filter,
            'exact': exact
        }

        response = self._http_client.get(self.__URL, headers=self._token_headers(token), json=payload)
        self._check_response(response)

        return DeviceMonitorReport(response.json())

        # return report.device(device_tag_filter) if exact else report


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "DeviceMonitorStatusManager:{}"



"""
Created on 24 Sep 2016

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

www.alphasense-technology.co.uk...
{"sensor_type":"PIDH2","serial_number":"143370503","we_electronic_zero_mv":"n/a","we_sensor_zero_mv":"61.88",
"we_total_zero_mv":"n/a","ae_electronic_zero_mv":"n/a","ae_sensor_zero_mv":"n/a","ae_total_zero_mv":"n/a",
"we_sensitivity_na_ppb":"n/a","we_cross_sensitivity_no2_na_ppb":"n/a","pcb_gain":"n/a",
"we_sensitivity_mv_ppb":"51.68","we_cross_sensitivity_no2_mv_ppb":"n/a","calibration_date":"2022-05-10"}
"""

from collections import OrderedDict

from scs_core.data.datum import Datum
from scs_core.gas.sensor_calib import SensorCalib


# --------------------------------------------------------------------------------------------------------------------

class PIDCalib(SensorCalib):
    """
    classdocs
    """

    @classmethod
    def construct_from_jdict(cls, jdict, skeleton=False):
        if not jdict:
            return None

        serial_number = jdict.get('serial_number')
        sensor_type = jdict.get('sensor_type')

        if 'pid_zero_mv' in jdict:
            pid_elc_mv = jdict.get('pid_zero_mv')
        else:
            pid_elc_mv = jdict.get('we_sensor_zero_mv')

        if 'pid_sensitivity_mv_ppm' in jdict:
            pid_sens_mv_ppm = jdict.get('pid_sensitivity_mv_ppm')
        else:
            pid_sens_mv_ppm = jdict.get('we_sensitivity_mv_ppb')        # uses mV / ppb field to record mV / ppm

        return cls(serial_number, sensor_type, pid_elc_mv, pid_sens_mv_ppm)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, serial_number, sensor_type, pid_elc_mv, pid_sens_mv_ppm):
        """
        Constructor
        """
        SensorCalib.__init__(self, serial_number, sensor_type)

        self.__pid_elc_mv = Datum.float(pid_elc_mv, 3)                  # PID electronic zero           mV
        self.__pid_sens_mv_ppm = Datum.float(pid_sens_mv_ppm, 6)        # PID sensitivity               mV / ppm

        self.validate()


    def __eq__(self, other):
        try:
            return self.serial_number == other.serial_number and self.sensor_type == other.sensor_type and \
                   self.pid_elc_mv == other.pid_elc_mv and self.pid_sens_mv_ppm == other.pid_sens_mv_ppm

        except (TypeError, AttributeError):
            return False


    # ----------------------------------------------------------------------------------------------------------------

    def validate(self):
        if self.pid_sens_mv_ppm == 0.0:
            raise ValueError('%s - pid_sensitivity_mv_ppm: zero sensitivity' % self.sensor_type)


    # ----------------------------------------------------------------------------------------------------------------

    def set_defaults(self):
        pass                                # PID may have null values


    def set_sens_mv_from_sens_na(self):
        pass                                # sensitivity nA data is not available


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['serial_number'] = self.serial_number
        jdict['sensor_type'] = self.sensor_type

        jdict['pid_zero_mv'] = self.pid_elc_mv
        jdict['pid_sensitivity_mv_ppm'] = self.pid_sens_mv_ppm

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def pid_elc_mv(self):
        return self.__pid_elc_mv


    @property
    def pid_sens_v_ppb(self):
        return self.__pid_sens_mv_ppm / 1000000.0


    @property
    def pid_sens_mv_ppm(self):
        return self.__pid_sens_mv_ppm


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "PIDCalib:{serial_number:%s, sensor_type:%s, pid_elc_mv:%s, pid_sens_mv_ppm:%s}" % \
                    (self.serial_number, self.sensor_type, self.pid_elc_mv, self.pid_sens_mv_ppm)



"""
Created on 24 Nov 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.security.cognito_user import CognitoUserIdentity

from scs_core.data.json import JSONify


# --------------------------------------------------------------------------------------------------------------------

class CognitoUserFinder(APIClient):
    """
    classdocs
    """

    __URL = 'https://iw0jza59y1.execute-api.us-west-2.amazonaws.com/default/CognitoUsersFinder/'

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def find_all(self, token):
        url = '/'.join((self.__URL, 'all'))

        response = self._http_client.get(url, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoUserIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def find_by_status(self, token, confirmation_status):
        url = '/'.join((self.__URL, confirmation_status.lower()))

        response = self._http_client.get(url, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoUserIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def find_by_enabled(self, token, enabled):
        url = '/'.join((self.__URL, 'enabled' if enabled else 'disabled'))

        response = self._http_client.get(url, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoUserIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def find_by_email(self, token, email):
        url = '/'.join((self.__URL, 'in'))
        payload = JSONify.dumps({"Email": email})

        response = self._http_client.get(url, data=payload, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoUserIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def find_by_usernames(self, token, usernames):
        url = '/'.join((self.__URL, 'usernames'))
        payload = JSONify.dumps(usernames)

        response = self._http_client.get(url, data=payload, headers=self._token_headers(token))
        self._check_response(response)

        return tuple(CognitoUserIdentity.construct_from_jdict(jdict) for jdict in response.json())


    def get_by_email(self, token, email):
        url = '/'.join((self.__URL, 'exact'))
        payload = JSONify.dumps({"Email": email})

        response = self._http_client.get(url, data=payload, headers=self._token_headers(token))
        self._check_response(response)

        return CognitoUserIdentity.construct_from_jdict(response.json())


    def get_self(self, token):
        url = '/'.join((self.__URL, 'self'))

        response = self._http_client.get(url, headers=self._token_headers(token))
        self._check_response(response)

        return CognitoUserIdentity.construct_from_jdict(response.json())



"""
Created on 5 Apr 2022

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.security.cognito_device import CognitoDeviceIdentity

from scs_core.data.json import JSONify


# --------------------------------------------------------------------------------------------------------------------

class CognitoDeviceManager(APIClient):
    """
    classdocs
    """

    __URL = 'https://6c2sfqt656.execute-api.us-west-2.amazonaws.com/default/CognitoDevices'

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def create(self, token, identity):
        response = self._http_client.post(self.__URL, headers=self._token_headers(token), data=JSONify.dumps(identity))
        self._check_response(response)

        return CognitoDeviceIdentity.construct_from_jdict(response.json())


    def update(self, token, identity):
        response = self._http_client.patch(self.__URL, headers=self._token_headers(token), data=JSONify.dumps(identity))
        self._check_response(response)

        return CognitoDeviceIdentity.construct_from_jdict(response.json())



    def delete(self, token, device_tag):
        payload = {"DeviceTag": device_tag}

        response = self._http_client.delete(self.__URL, headers=self._token_headers(token), data=JSONify.dumps(payload))
        self._check_response(response)

        # TODO: delete device from organisations?


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CognitoDeviceManager:{}"



"""
Created on 24 Apr 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

Enables a device to register itself as a CognitoDevice
"""

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.security.cognito_device import CognitoDeviceIdentity

from scs_core.data.json import JSONify


# --------------------------------------------------------------------------------------------------------------------

class CognitoDeviceCreator(APIClient):
    """
    classdocs
    """

    __URL = 'https://bf32xbgymi.execute-api.us-west-2.amazonaws.com/default/CognitoDeviceCreator'
    __AUTH = "@southcoastscience.com"

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def create(self, identity: CognitoDeviceIdentity):
        response = self._http_client.post(self.__URL, headers=self._auth_headers(self.__AUTH),
                                          data=JSONify.dumps(identity))
        self._check_response(response)

        return CognitoDeviceIdentity.construct_from_jdict(response.json())


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "CognitoDeviceCreator:{}"



"""
Created on 17 Jun 2023

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_core.aws.client.api_client import APIClient

from scs_core.aws.data.device_monitor_email_list import DeviceMonitorEmailList


# --------------------------------------------------------------------------------------------------------------------

class DeviceMonitorSpecificationManager(APIClient):
    """
    classdocs
    """

    __URL = "https://psnunpg4gb.execute-api.us-west-2.amazonaws.com/default/DeviceMonitorSpecification"

    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def find(self, token, email_address=None, device_tag=None, exact=False):
        payload = {
            'email_address': email_address,
            'device_tag': device_tag,
            'exact': exact
        }

        response = self._http_client.get(self.__URL, headers=self._token_headers(token), json=payload)
        self._check_response(response)

        return DeviceMonitorEmailList.construct_from_jdict(response.json())


    def add(self, token, email_address, device_tag):
        payload = {
            'email_address': email_address,
            'device_tag': device_tag
        }

        response = self._http_client.post(self.__URL, headers=self._token_headers(token), json=payload)
        self._check_response(response)

        return DeviceMonitorEmailList.construct_from_jdict(response.json())


    def remove(self, token, email_address, device_tag=None):
        payload = {
            'email_address': email_address,
            'device_tag': device_tag
        }

        response = self._http_client.delete(self.__URL, headers=self._token_headers(token), json=payload)
        self._check_response(response)

        return DeviceMonitorEmailList.construct_from_jdict(response.json())


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "DeviceMonitorSpecificationManager:{}"



"""
Created on 23 Nov 2021

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from collections import OrderedDict
from enum import Enum

from scs_core.aws.client.api_client import APIClient
from scs_core.aws.data.http_response import HTTPResponse

from scs_core.data.json import JSONable


# --------------------------------------------------------------------------------------------------------------------

class CognitoLoginManager(APIClient):
    """
    classdocs
    """

    __URL = 'https://lnh2y9ip75.execute-api.us-west-2.amazonaws.com/default/CognitoLogin'
    __AUTH = '@southcoastscience.com'


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_client):
        super().__init__(http_client)


    # ----------------------------------------------------------------------------------------------------------------

    def user_login(self, credentials):
        url = '/'.join((self.__URL, 'user'))
        headers = self._auth_headers(self.__AUTH)

        response = self._http_client.post(url, headers=headers, json=credentials.as_json())
        self._check_response(response)

        return AuthenticationResult.construct_from_res(response)


    def device_login(self, credentials):
        url = '/'.join((self.__URL, 'device'))
        headers = self._auth_headers(self.__AUTH)

        response = self._http_client.post(url, headers=headers, json=credentials.as_json())
        self._check_response(response)

        return AuthenticationResult.construct_from_res(response)


# --------------------------------------------------------------------------------------------------------------------

class AuthenticationResult(HTTPResponse):
    """
    {"AccessToken": "...",
    "ExpiresIn": 3600,
    "TokenType": "Bearer",
    "RefreshToken": "...",
    "IdToken": "..."}
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_response_jdict(cls, status, jdict):
        if not jdict:
            return None

        # print(json.dumps(jdict, indent=4))

        authentication_status = AuthenticationStatus.construct_from_jdict(jdict.get('authentication-status'))

        if authentication_status == AuthenticationStatus.Ok:
            content = Session.construct_from_jdict(jdict.get('content').get('AuthenticationResult'))

        elif authentication_status == AuthenticationStatus.Challenge:
            content = Challenge.construct_from_jdict(jdict.get('content'))

        else:
            content = None

        return cls(status, authentication_status, content)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, http_status, authentication_status, content):
        """
        Constructor
        """
        super().__init__(http_status)

        self.__authentication_status = authentication_status            # AuthenticationStatus
        self.__content = content                                        # Session, Challenge or None


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['authentication-status'] = self.authentication_status
        jdict['content'] = self.content

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    def is_ok(self):
        return self.authentication_status == AuthenticationStatus.Ok


    @property
    def id_token(self):
        if not self.is_ok():
            raise ValueError(self.authentication_status)

        return self.content.id_token


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def authentication_status(self):
        return self.__authentication_status


    @property
    def content(self):
        return self.__content


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "AuthenticationResult:{http_status:%s, authentication_status:%s, content:%s}" % \
               (self.status, self.authentication_status, self.content)


# --------------------------------------------------------------------------------------------------------------------

class AuthenticationStatus(JSONable, Enum):
    """
    classdocs
    """

    Ok = (True, 'OK.')
    InvalidCredentials = (False, 'Invalid credentials.')
    Challenge = (False, 'Authentication challenge.')
    UserUnconfirmed = (False, 'User is unconfirmed.')


    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        return AuthenticationStatus[jdict]


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, ok, description):
        self.__ok = ok                                      # bool
        self.__description = description                    # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        return self.name


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def ok(self):
        return self.__ok


    @property
    def description(self):
        return self.__description


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "AuthenticationStatus:{ok:%s, description:%s}" %  (self.ok, self.description)


# --------------------------------------------------------------------------------------------------------------------

class Challenge(JSONable):
    """
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        challenge_name = jdict.get('ChallengeName')
        session = jdict.get('Session')

        return cls(challenge_name, session)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, challenge_name, session):
        """
        Constructor
        """
        self.__challenge_name = challenge_name                  # string
        self.__session = session                                # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['ChallengeName'] = self.challenge_name
        jdict['Session'] = self.session

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def challenge_name(self):
        return self.__challenge_name


    @property
    def session(self):
        return self.__session


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "Challenge:{challenge_name:%s, session:%s}" % \
               (self.challenge_name, self.session)


# --------------------------------------------------------------------------------------------------------------------

class Session(JSONable):
    """
    {"AccessToken": "...",
    "ExpiresIn": 3600,
    "TokenType": "Bearer",
    "RefreshToken": "...",
    "IdToken": "..."}
    """

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_from_jdict(cls, jdict):
        if not jdict:
            return None

        access_token = jdict.get('AccessToken')
        expires_in = jdict.get('ExpiresIn')
        token_type = jdict.get('TokenType')
        refresh_token = jdict.get('RefreshToken')
        id_token = jdict.get('IdToken')

        return cls(access_token, expires_in, token_type, refresh_token, id_token)


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, access_token, expires_in, token_type, refresh_token, id_token):
        """
        Constructor
        """
        self.__access_token = access_token                      # string
        self.__expires_in = int(expires_in)                     # int
        self.__token_type = token_type                          # string
        self.__refresh_token = refresh_token                    # string
        self.__id_token = id_token                              # string


    # ----------------------------------------------------------------------------------------------------------------

    def as_json(self):
        jdict = OrderedDict()

        jdict['AccessToken'] = self.access_token
        jdict['ExpiresIn'] = self.expires_in
        jdict['TokenType'] = self.token_type
        jdict['RefreshToken'] = self.refresh_token
        jdict['IdToken'] = self.id_token

        return jdict


    # ----------------------------------------------------------------------------------------------------------------

    @property
    def access_token(self):
        return self.__access_token


    @property
    def expires_in(self):
        return self.__expires_in


    @property
    def token_type(self):
        return self.__token_type


    @property
    def refresh_token(self):
        return self.__refresh_token


    @property
    def id_token(self):
        return self.__id_token


    # ----------------------------------------------------------------------------------------------------------------

    def __str__(self, *args, **kwargs):
        return "Session:{access_token:%s, expires_in:%s, token_type:%s, refresh_token:%s, id_token:%s}" % \
               (self.access_token, self.expires_in, self.token_type, self.refresh_token, self.id_token)
